%
% MATLAB_EM_Plot_DC_Shunt_Speed_Torque.m
% for MATLAB R2018a Update 6 (9.4.0.949201) 64bit (win64) Sept 5,2018
%
% V1.0 - Ben Rowland 12 August 2019
%
% Plot the DC Shunt Motor Torque v Speed characteristics
%
% Note:
% use pathtool or Set Path
% to point to your working directory and include the ~/Primitives directory

clear all;

% include the ~/Primitives directory
addpath("Primitives");

% This forces the COM port to be closed, otherwise it can become locked
% Requires the Instrument control toolkit to allow this to function
delete(instrfindall)
clear EM_USB;
instrreset;

% Global variable holding the reference to the COM port, used by our
% Primitives library
global EM_USB;

% Find the COM port on your PC where the Matrix EM control unit is plugged
% in and use it here instead of 'COM17'
EM_USB = serial('COM5','Baudrate',115200,'Databits',8);
fopen(EM_USB);

% Early versions of the EM hardware may require EM_VERSION to be 0
% if comms is not working then try changing to 0
global EM_VERSION;
EM_VERSION = 1;

% This just tests the comms to the control unit before we start
api_version = GetAPIVersion();
disp(['api_version = ' , num2str(api_version)]);


% Preconditions
SetActive(0);         % Motor OFF
SetCapacitorBank(0);  % OFF
SetDynoLoad(127); % Max Resistance = Minimum Load
SetMotorDirection(0); % Clockwise
SetMotorType (0);     % DC Motor
ZeroWeight();

motor_voltage = 16;   % 16V DC
dyno_load = 127;      % Dynamometer Load Resistance = 127 Ohms


% This just tests the comms to the control unit before we start
api_version = GetAPIVersion();
disp(['api_version = ' , num2str(api_version)]);

% Check the Dynamometer 'motor located' switches
switch_state = GetSwitches ();
if switch_state >= 1
   disp('==== DC MOTOR TEST START ====')
   disp('Motor Installed Correctly');
   current_I_DC = GetCurrent(0);
   disp(['DC Current with motor OFF = ' , num2str(current_I_DC) , 'A']);
   pause(0.1);
   disp(['Setting Motor Voltage to ' , num2str(motor_voltage) , ' Volts']);
   SetVoltage (0, motor_voltage);
   pause(0.1);
   disp(['Setting Dyno Load to ' , num2str(dyno_load) , ' Ohms']);
   SetDynoLoad(dyno_load);
   pause(0.1);
   current_I1 = GetCurrent(6);
   disp(['I1 Current (with Motor OFF) = ' , num2str(current_I1) , 'A']);    
   loop_count = 1000;
   disp('MOTOR ON');
   index = 1;
   SetActive(1); % Motor ON
   pause(0.3); % Wait for the motor to spin up

   % Perform the sampled measurements in as tight a loop as you can.
   for loop = 1:127
      load_cell_torque_range(index) = GetLoadCell(1);
      encoder_speed_range(index) = GetRPM();
      disp(['Dyno Load = ' , num2str(dyno_load) , 'R - RPM = ' , num2str(encoder_speed_range(index)) , ' - Torque = ' , num2str(load_cell_torque_range(index))]);    

      dyno_load = 127 - loop; 
      SetDynoLoad(loop);

      index = index + 1;
      pause(0.5);
   end

   % Sampling over, you can turn off the motor
   % and close the COM port
   disp('MOTOR OFF');
   SetActive(0); % Motor OFF
   pause(0.1);
   fclose(EM_USB)
  
   % Plot the RPM against Time
   curent_torque_plot = figure;
   figure(curent_torque_plot);
   movegui(curent_torque_plot , 'northwest');
   hold on
   p2 = polyfit(load_cell_torque_range, encoder_speed_range, 3); % curve of best fit
   y2 = polyval(p2, load_cell_torque_range);
   plot(load_cell_torque_range, encoder_speed_range, 'ro', load_cell_torque_range, y2, '-r' );
   torque_speed_polynomial = poly2sym(p2,sym('RPM'));
   torque_speed_equation = sprintf('Torque = %s',torque_speed_polynomial); 
   disp(torque_speed_equation);
   grid on
   legend('Samples','Best fit curve','Location','northeast');
   xlabel('Torque, mNm');
   ylabel('Speed, RPM');
   title(['DC Shunt Motor Torque v Speed, with Armature @ ', num2str(motor_voltage) , 'V and Stator @ 24V']);      
   hold off
   
   pause(0.1);
   disp('==== DC MOTOR TEST END ======');
else    
   disp('==== Motor not fitted in Dynomometer ====');
end


% Check for corrupt comms
fopen(EM_USB);
api_version_two = GetAPIVersion();
if api_version == api_version_two
   % do nothing
else
    disp('=== CORRUPT COMMS ===');
    disp(['api_version = ' , num2str(api_version_two) , ' it was ' , num2str(api_version) ] );
end
   
fclose(EM_USB) % Close the COM port

% MATLAB_EM_Plot_DC_Shunt_Torque_Speed.m


